/* $Id: options.c,v 1.5 2000/09/11 00:01:55 bodo Exp $ */
/*
 * This file is part of relay.
 *
 * Copyright (C) 2000 Bodo Bellut
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Authors:  Bodo Bellut (bodo@bellut.net)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>

#include <options.h>

char *progname = NULL;


void usage(char *argv, char *nextargv)
{
  if (nextargv)
    fprintf(stderr, "illegal value %s to option %s\n", nextargv, argv);
  else if (argv)
    fprintf(stderr, "unknown option: %s\n", argv);
    
  fprintf(stderr, "usage: %s <options>\n", progname);
  
  fprintf(stderr, "Available options:\n");
  fprintf(stderr, "\t--\t\tset all relays \"off\"\n");
  fprintf(stderr, "\t++\t\tset all relays \"on\"\n");
  fprintf(stderr, "\t-<n>\t\tset relay <n> \"off\"\n");
  fprintf(stderr, "\t+<n>\t\tset relay <n> \"on\"\n");
  fprintf(stderr, "\t-e\t\tdon't Execute broadcasts\n");
  fprintf(stderr, "\t+e\t\tExecute broadcasts\n");
  fprintf(stderr, "\t-b\t\tdon't Block broadcasts\n");
  fprintf(stderr, "\t+b\t\tBlock broadcasts\n");
  fprintf(stderr, "\t-s\t\tshow relay Status\n");
  fprintf(stderr, "\t-o\t\tshow Options\n");
  fprintf(stderr, "\t-n [<n>]\tshow status Numerically\n");
  fprintf(stderr, "\t\t\tset status Numerically (to <n>)\n");
  fprintf(stderr, "\t-c <n>\t\ttalk to Card <n>\n");
  fprintf(stderr, "\t\t\tdefaults to 1, use 0 for broadcast\n");
  fprintf(stderr, "\t--port <s>\tuse port <s>\n");
  fprintf(stderr, "\t\t\tdefaults to $RELAYPORT or /dev/ttyS0\n\n");
  fprintf(stderr, "There are also some long options:\n");
  fprintf(stderr, "\t--num\t\tequivalent to -n\n");
  fprintf(stderr, "\t--show-status\tequivalent to -s\n");
  fprintf(stderr, "\t--show-opts\tequivalent to -o\n");
  fprintf(stderr, "\t--card\t\tequivalent to -c\n");
  
}

char process_longopt(char *argv, char *nextargv, options *opt)
{
  char *p = argv;
  char *err = NULL;
  
  p +=2;  /* skip "--" */
  
  if (!strcasecmp(p, "show-status"))
    opt->showstatus = 1;
  else if (!strcasecmp(p, "show-opts"))
    opt->showopts = 1;
  else if (!strcasecmp(p, "num")) {
    opt->numerically = 1;
    if (nextargv && nextargv[0] != '-') {
      opt->setrelay = strtoul(nextargv, &err, 0);
      if (*nextargv != '\0' && *err == '\0') {
        return 1;
      } else {
        usage(argv, nextargv);
        exit(-1);
      }
    }
  } else if (!strcasecmp(p, "port")) {
    if (nextargv && nextargv[0] != '-' && nextargv[0] != '+') {
      strncpy(opt->port, nextargv, sizeof(opt->port));
      return 1;
    } else {
      usage(argv, nextargv);
      exit(-1);
    }
  } else if (!strcasecmp(p, "card")) {
    if (nextargv && nextargv[0] != '-') {
    
      if (!strcasecmp(nextargv, "all")) {
        opt->card = 0;
        return 1;
      } else {
        opt->card = strtoul(nextargv, &err, 0);
        if (*nextargv != '\0' && *err == '\0') {
          return 1;
        } else {
          usage(argv, nextargv);
          exit(-1);
        }
      }
    } else {
      usage(argv, nextargv);
      exit(-1);
    }
  } else {
    usage(argv, NULL);
    exit(-1);
  }
  
  return 0;
  
}

void init_options(options *opt)
{
  char *port = getenv("RELAYPORT");

  opt->clearall = 0;
  opt->setall = 0;
  memset(opt->clear, 0, sizeof(opt->clear));
  memset(opt->set,   0, sizeof(opt->set));
  opt->clearexec = 0;
  opt->clearblock = 0;
  opt->setexec = 0;
  opt->setblock = 0;
  opt->showstatus = 0;
  opt->showopts = 0;
  opt->setrelay = -1L;
  opt->numerically = 0;
  if (port)
    strncpy(opt->port, port, sizeof(opt->port));
  else
    strncpy(opt->port, DEFAULTPORT, sizeof(opt->port));
  opt->card = 1;
  opt->individual = 0;
}

void process_options(int argc, char *argv[], options *opt)
{
  int i = 1;
  char *err = NULL;
  
  init_options(opt);
  
  progname = argv[0];
  
  if (argc <= 1) {
    usage(NULL, NULL);
    exit(-1);
  }
  
  while (argc > 1) {
    
    switch(argv[i][0]) {
    case '-':
      if (argv[i][1] == '-') {      /* -- */
        if (argv[i][2] == '\0') {
          opt->clearall = 1;
        } else {
          if (process_longopt(argv[i],argv[i+1]?argv[i+1]:NULL, opt)) {
            argc--;
            i++;
          }
        }
      } else {                      /* -<...> */
        if (argv[i][2] != '\0') {
          usage(argv[i], NULL);
          exit (-1);
        }
        switch(argv[i][1]) {
        case '1': opt->clear[0] = 1; opt->individual = 1; break;
        case '2': opt->clear[1] = 1; opt->individual = 1; break;
        case '3': opt->clear[2] = 1; opt->individual = 1; break;
        case '4': opt->clear[3] = 1; opt->individual = 1; break;
        case '5': opt->clear[4] = 1; opt->individual = 1; break;
        case '6': opt->clear[5] = 1; opt->individual = 1; break;
        case '7': opt->clear[6] = 1; opt->individual = 1; break;
        case '8': opt->clear[7] = 1; opt->individual = 1; break;
        case 'e': opt->clearexec = 1; break;
        case 'b': opt->clearblock = 1; break;
        case 's': opt->showstatus = 1; break;
        case 'o': opt->showopts = 1; break;
        case 'n': 
          opt->numerically = 1;
          if (i+1 < argc && argv[i+1][0] != '-') {
            opt->setrelay = strtoul(argv[i+1], &err, 0);
            if (*argv[i+1] != '\0' && *err == '\0') {
              argc--;
              i++;
            } else {
              usage(argv[i], argv[i+1]);
              exit(-1);
            }
          }
          break;
        case 'c':
          if (i+1 < argc && argv[i+1][0] != '-') {
            opt->card = strtoul(argv[i+1], &err, 0);
            if (*argv[i+1] != '\0' && *err == '\0') {
              argc--;
              i++;
            } else {
              usage(argv[i], argv[i+1]);
              exit(-1);
            }
          } else {
            usage(argv[i], argv[i+1]);
            exit(-1);
          }
          break;
        default: usage(argv[i], NULL); exit(-1); break;
        }
      }
      break;
    case '+':
      if (argv[i][1] == '+') {      /* ++ */
        if (argv[i][2] == '\0') {
          opt->setall = 1;
        } else {
          usage(argv[i], NULL);
          exit(-1);
        }
      } else {                      /* +<...> */
        if (argv[i][2] != '\0') {
          usage(argv[i], NULL);
          exit (-1);
        }
        switch(argv[i][1]) {
        case '1': opt->set[0] = 1; opt->individual = 1; break;
        case '2': opt->set[1] = 1; opt->individual = 1; break;
        case '3': opt->set[2] = 1; opt->individual = 1; break;
        case '4': opt->set[3] = 1; opt->individual = 1; break;
        case '5': opt->set[4] = 1; opt->individual = 1; break;
        case '6': opt->set[5] = 1; opt->individual = 1; break;
        case '7': opt->set[6] = 1; opt->individual = 1; break;
        case '8': opt->set[7] = 1; opt->individual = 1; break;
        case 'e': opt->setexec = 1; break;
        case 'b': opt->setblock = 1; break;
        default: usage(argv[i], NULL); exit(-1); break;
        }
      }
      break;
    default: usage(argv[i], NULL); exit(-1); break;
    }
    
    i++;
    argc--;
  }

  if (opt->clearall || opt->setall) {
    if (opt->showopts || opt->showstatus ||
        opt->numerically || opt->individual) {
      usage(NULL, NULL);
      exit(-1);
    }
  }
  
  if (opt->clearall && opt->setall) {
    usage(NULL, NULL);
    exit(-1);
  }

  if ((opt->clearexec && opt->setexec) || (opt->clearblock && opt->setblock)) {
    usage(NULL, NULL);
    exit(-1);
  }
  
  if (opt->numerically &&
     (opt->clearexec || opt->clearblock ||
      opt->setexec   || opt->setblock   ||
      opt->showstatus || opt->showopts)) {
    usage(NULL, NULL);
    exit(-1);
  }
  {
  int i;
  for (i=0; i < 8; i++) {
    if (opt->clear[i] && opt->set[i]) {
      usage(NULL, NULL);
      exit(-1);
    }
  }
  }
  if (opt->individual &&
     (opt->showstatus || opt->showopts || opt->numerically)) {
    
    usage(NULL, NULL);
    exit(-1);
  }
  
}
