/* $Id: relay.c,v 1.5 2000/09/11 00:01:55 bodo Exp $ */
/*
 * This file is part of relay.
 *
 * Copyright (C) 2000 Bodo Bellut
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Authors:  Bodo Bellut (bodo@bellut.net)
 */


#include <config.h>

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/time.h>
#include <string.h>

#include <relay.h>

struct termios backup;


int senddata(int fd, rdata data) {
 data[3] = data[0] ^ data[1] ^ data[2];
 write(fd, data, 4);
 return RELAIS_TRUE;
}

int readdata(int fd, rdata data) {    
 if (read(fd, data, 4)<4) return RELAIS_ERROR;
 return ((data[0] ^ data[1] ^ data[2]) == data[3])?RELAIS_TRUE:RELAIS_FALSE;
}

int initcards(int fd, int timeout)
{
 struct termios newtio;
 rdata relaydata;
 int status;
 
 tcgetattr(fd, &backup);

 bzero(&newtio, sizeof(newtio));
 newtio.c_cflag = B19200 | CS8 | CLOCAL | CREAD;
 newtio.c_iflag = IGNPAR;
 newtio.c_oflag = 0;
 newtio.c_lflag = 0;

 newtio.c_cc[VTIME]    = timeout*10; 
 newtio.c_cc[VMIN]     = 0; 

 tcflush(fd, TCIFLUSH);
 tcsetattr(fd,TCSANOW,&newtio);
 
 relaydata[0]= 1; relaydata[1]= 1; relaydata[2]= 0;
 senddata(fd, relaydata);
 do {
  if ((status=readdata(fd, relaydata))==RELAIS_ERROR) break;
 } while (relaydata[0]!=1);
 flushfd(fd);
 return status>0&&relaydata[0]==1?relaydata[1]-1:0;
}

int restoretty(int fd)
{
 tcflush(fd, TCIFLUSH);
 tcsetattr(fd,TCSANOW,&backup);
 return RELAIS_TRUE;
} 

int nop(int fd, int card)
{
 rdata relaydata;

 relaydata[0]= 0; relaydata[1]= card;
 senddata(fd, relaydata);
 do {
  if (readdata(fd, relaydata)==RELAIS_ERROR) break;
 } while (relaydata[0]!=255);
 flushfd(fd);
 return relaydata[0]==255?RELAIS_TRUE:RELAIS_ERROR;
}

int setrelay(int fd, int card, int relay)
{
 rdata relaydata;
 
 relaydata[0]= 3; relaydata[1]= card; relaydata[2]= relay;
 senddata(fd, relaydata);
 do {
  if (readdata(fd, relaydata)==RELAIS_ERROR) break;
 } while (relaydata[0]!=252);
 flushfd(fd);
 return relaydata[0]== 252;
}

int getrelay(int fd, int card)
{
 rdata relaydata;

 relaydata[0]= 2; relaydata[1]= card;
 senddata(fd, relaydata);
 do {
  if (readdata(fd, relaydata)==RELAIS_ERROR) break;
 } while (relaydata[0]!=253);
 flushfd(fd);
 return relaydata[0]==253?relaydata[2]:RELAIS_ERROR;
} 
 
int setoption(int fd, int card, int opts)
{
 rdata relaydata;
 
 relaydata[0]= 5; relaydata[1]= card; relaydata[2]= opts;
 senddata(fd, relaydata);
 do {
  if (readdata(fd, relaydata)==RELAIS_ERROR) break;
 } while (relaydata[0]!=250);
 flushfd(fd);
 return relaydata[0]== 250;
}

int getoption(int fd, int card)
{
 rdata relaydata;

 relaydata[0]= 4; relaydata[1]= card;
 senddata(fd, relaydata);
 do {
  if (readdata(fd, relaydata)==RELAIS_ERROR) break;
 } while (relaydata[0]!=251);
 flushfd(fd);
 return relaydata[0]==251?relaydata[2]:RELAIS_ERROR;
} 
